﻿
using System;
namespace HeuristicLab.DataPreprocessing {

  public class TransformationLogic : ITransformationLogic {

    private readonly IPreprocessingData preprocessingData;
    private readonly ISearchLogic searchLogic;
    private readonly IStatisticsLogic statisticsLogic;

    public TransformationLogic(IPreprocessingData thePreprocessingData, ISearchLogic theSearchLogic, IStatisticsLogic theStatisticsLogic) {
      preprocessingData = thePreprocessingData;
      searchLogic = theSearchLogic;
      statisticsLogic = theStatisticsLogic;
    }

    public void deleteRowsWithMissingValuesGreater(double percent) {
      for (int i = 0; i < preprocessingData.Rows; ++i) {
        int missingCount = statisticsLogic.GetRowMissingValueCount(i);
        if (100f / preprocessingData.Columns * missingCount >= percent) {
          preprocessingData.DeleteRow(i);
          --i;
        }
      }
    }

    public void deleteColumnsWithMissingValuesGreater(float percent) {
      for (int i = 0; i < preprocessingData.Columns; ++i) {
        int missingCount = statisticsLogic.GetMissingValueCount(i);
        if (100f / preprocessingData.Columns * missingCount >= percent) {
          preprocessingData.DeleteColumn(i);
          --i;
        }
      }
    }

    public void deleteColumnsWithVarianceSmaller(double variance) {
      for (int i = 0; i < preprocessingData.Columns; ++i) {
        if (preprocessingData.IsType<double>(i) || preprocessingData.IsType<DateTime>(i)) {
          double columnVariance = statisticsLogic.GetVariance(i);
          if (columnVariance < variance) {
            preprocessingData.DeleteColumn(i);
            --i;
          }
        }
      }
    }

  }
}
